
# q-expansion of newform 50.6.b.d, downloaded from the LMFDB on 06 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 50
weight = 6
poly_data = [1, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 2], 1]]

hecke_ring_character_values = [[27, [-1, 0]]]
aps_data = [[0, 2], [0, -13], [0, 0], [0, 11], [-768, 0], [0, -23], [0, -189], [-1100, 0], [0, -993], [5610, 0], [-3988, 0], [0, 71], [1542, 0], [0, -2513], [0, -12369], [0, -7083], [-28380, 0], [5522, 0], [0, 12371], [42372, 0], [0, -26063], [39640, 0], [0, -29913], [-57690, 0], [0, 72191], [-141258, 0], [0, 69907], [0, -43209], [-218450, 0], [0, -14403], [0, 108251], [-244608, 0], [0, 119751], [-30860, 0], [100950, 0], [12452, 0], [0, 3011], [0, -250433], [0, -277629], [0, 208677], [52380, 0], [546662, 0], [-452028, 0], [0, 242797], [0, -505089], [807640, 0], [149552, 0], [0, -221753], [0, -210009], [-1058750, 0], [0, -636723], [370680, 0], [-561298, 0], [577152, 0], [0, 325731], [0, 458787], [735390, 0], [-1121308, 0], [0, 830171], [1452102, 0], [0, 154507], [0, -796503], [0, -623629], [-665988, 0], [0, -295643], [0, 258171], [-3295768, 0], [0, -955489], [0, -1210029], [-2507270, 0], [0, -206583], [-1737720, 0], [0, -580489], [0, 171877], [-573140, 0], [0, -1440273], [3085590, 0], [0, -442729], [-3753438, 0], [1946530, 0], [2991660, 0], [3966602, 0], [-5171148, 0], [0, -2267423], [1082200, 0], [0, -540393], [-2617830, 0], [0, -795229], [4254702, 0], [0, 1633027], [0, 300771], [4579320, 0], [0, -3526129], [-2623488, 0], [3612340, 0], [0, 4578147], [-7261590, 0], [5810202, 0], [0, -4085333], [-817378, 0], [0, 1753751], [0, -4807449], [0, 1009707], [-1378590, 0], [8542952, 0], [0, 1157291], [0, -464169], [0, -454743], [8511360, 0], [6124982, 0], [0, 2258231], [0, 4819897], [0, 4963251], [-7633220, 0], [18031412, 0], [9301902, 0], [0, -6916613], [0, 7449831], [0, -9665283], [4061100, 0], [-13515238, 0], [0, 7175977], [0, -947649], [0, 1458207], [14427752, 0], [-15867918, 0], [301810, 0], [-21267720, 0], [0, -7750429], [0, -6092483], [29028220, 0], [0, 8057247], [-2924308, 0], [0, -13016269], [16326282, 0], [-25813250, 0], [0, -9529623], [0, 8670551], [0, 12908451], [-8884890, 0], [-24639568, 0], [11376762, 0], [0, -6539153], [0, 17859411], [-16188050, 0], [25549680, 0], [0, -11147663], [0, -9843609], [-6775820, 0], [0, -13190193], [0, -14758069], [-14856498, 0], [0, -7267013], [0, 8601411], [0, 17221691], [-983748, 0], [-30885680, 0], [32087370, 0], [0, -7625989], [3481662, 0], [0, 12700491], [0, -24867603], [0, -15258649], [31985352, 0], [0, -1453929], [0, 17481027], [3004652, 0], [0, -16039429]]
